@@ -19,19 +19,30 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import traceback
-from collections import deque
 from PyQt4 import QtCore, QtNetwork
-from picard.metadata import Metadata, run_album_metadata_processors, run_track_metadata_processors
+from picard.coverart import coverart
+from picard.metadata import (Metadata,
+                             register_album_metadata_processor,
+                             run_album_metadata_processors,
+                             run_track_metadata_processors)
 from picard.dataobj import DataObject
 from picard.file import File
 from picard.track import Track
 from picard.script import ScriptParser
 from picard.ui.item import Item
-from picard.util import format_time, partial, queue, mbid_validate, asciipunct
+from picard.util import format_time, queue, mbid_validate, asciipunct
 from picard.cluster import Cluster
-from picard.mbxml import release_to_metadata, medium_to_metadata, track_to_metadata, media_formats_from_node, label_info_from_node
+from picard.collection import Collection, user_collections, load_user_collections
+from picard.mbxml import (
+    release_group_to_metadata,
+    release_to_metadata,
+    medium_to_metadata,
+    track_to_metadata,
+)
 from picard.const import VARIOUS_ARTISTS_ID
 
+register_album_metadata_processor(coverart)
+
 
 class Album(DataObject, Item):
 
@@ -41,17 +52,14 @@ class Album(DataObject, Item):
         DataObject.__init__(self, id)
         self.metadata = Metadata()
         self.tracks = []
-        self.format_str = ""
         self.loaded = False
         self.load_task = None
-        self.rgloaded = False
-        self.rgid = None
+        self.release_group = None
         self._files = 0
         self._requests = 0
         self._tracks_loaded = False
         self._discid = discid
         self._after_load_callbacks = queue.Queue()
-        self.other_versions = []
         self.unmatched_files = Cluster(_("Unmatched Files"), special=True, related_album=self, hide_if_empty=True)
 
     def __repr__(self):
@@ -87,10 +95,16 @@ class Album(DataObject, Item):
         # Get release metadata
         m = self._new_metadata
         m.length = 0
+
+        rg_node = release_node.release_group[0]
+        rg = self.release_group = self.tagger.get_release_group_by_id(rg_node.id)
+        rg.loaded_albums.add(self.id)
+        rg.refcount += 1
+
+        release_group_to_metadata(rg_node, rg.metadata, self.config, rg)
+        m.copy(rg.metadata)
         release_to_metadata(release_node, m, config=self.config, album=self)
 
-        self.format_str = media_formats_from_node(release_node.medium_list[0])
-        self.rgid = release_node.release_group[0].id
         if self._discid:
             m['musicbrainz_discid'] = self._discid
 
@@ -104,6 +118,15 @@ class Album(DataObject, Item):
 
         m['totaldiscs'] = release_node.medium_list[0].count
 
+        # Add album to collections
+        if "collection_list" in release_node.children:
+            for node in release_node.collection_list[0].collection:
+                if node.editor[0].text.lower() == self.config.setting["username"].lower():
+                    if node.id not in user_collections:
+                        user_collections[node.id] = \
+                            Collection(node.id, node.name[0].text, node.release_list[0].count)
+                    user_collections[node.id].releases.add(self.id)
+
         # Run album metadata plugins
         try:
             run_album_metadata_processors(self, m, release_node)
@@ -146,34 +169,6 @@ class Album(DataObject, Item):
             if parsed or error:
                 self._finalize_loading(error)
 
-    def _parse_release_group(self, document):
-        for node in document.metadata[0].release_list[0].release:
-            v = {}
-            v["mbid"] = node.id
-            v["date"] = node.date[0].text if "date" in node.children else ""
-            v["country"] = node.country[0].text if "country" in node.children else ""
-            labels, catnums = label_info_from_node(node.label_info_list[0])
-            v["labels"] = ", ".join(set(labels))
-            v["catnums"] = ", ".join(set(catnums))
-            v["tracks"] = " + ".join([m.track_list[0].count for m in node.medium_list[0].medium])
-            v["format"] = media_formats_from_node(node.medium_list[0])
-            self.other_versions.append(v)
-        self.other_versions.sort(key=lambda x: x["date"])
-
-    def _release_group_request_finished(self, document, http, error):
-        try:
-            if error:
-                self.log.error("%r", unicode(http.errorString()))
-            else:
-                try:
-                    self._parse_release_group(document)
-                except:
-                    error = True
-                    self.log.error(traceback.format_exc())
-        finally:
-            self.rgloaded = True
-            self.release_group_loaded.emit()
-
     def _finalize_loading(self, error):
         if error:
             self.metadata.clear()
@@ -274,9 +269,9 @@ class Album(DataObject, Item):
             return
         self.tagger.window.set_statusbar_message('Loading album %s...', self.id)
         self.loaded = False
-        self.rgloaded = False
-        self.rgid = None
-        self.other_versions = []
+        if self.release_group:
+            self.release_group.loaded = False
+            self.release_group.folksonomy_tags.clear()
         self.metadata.clear()
         self.folksonomy_tags.clear()
         self.metadata['album'] = _("[loading album information]")
@@ -285,7 +280,8 @@ class Album(DataObject, Item):
         self._new_tracks = []
         self._requests = 1
         require_authentication = False
-        inc = ['release-groups', 'media', 'recordings', 'puids', 'artist-credits', 'artists', 'aliases', 'labels', 'isrcs']
+        inc = ['release-groups', 'media', 'recordings', 'artist-credits',
+               'artists', 'aliases', 'labels', 'isrcs', 'collections']
         if self.config.setting['release_ars'] or self.config.setting['track_ars']:
             inc += ['artist-rels', 'release-rels', 'url-rels', 'recording-rels', 'work-rels']
             if self.config.setting['track_ars']:
@@ -393,6 +389,9 @@ class Album(DataObject, Item):
     def can_refresh(self):
         return True
 
+    def can_view_info(self):
+        return bool(self.loaded and self.metadata and self.metadata.images)
+
     def is_album_like(self):
         return True
 
@@ -437,6 +436,8 @@ class Album(DataObject, Item):
                 unsaved = self.get_num_unsaved_files()
                 if unsaved:
                     text += '; %d*' % (unsaved,)
+                text += ungettext("; %i image", "; %i images",
+                        len(self.metadata.images)) % len(self.metadata.images)
                 return text + ')'
             else:
                 return self.metadata['album']
@@ -463,6 +464,7 @@ class Album(DataObject, Item):
             self.tagger.remove_album(self)
         else:
             del self.tagger.albums[self.id]
+            self.release_group.loaded_albums.discard(self.id)
             self.id = mbid
             self.tagger.albums[mbid] = self
             self.load()
