@@ -19,6 +19,7 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import re
+from operator import itemgetter
 from heapq import heappush, heappop
 from PyQt4 import QtCore
 from picard.metadata import Metadata
@@ -33,7 +34,7 @@ class Cluster(QtCore.QObject, Item):
     # Weights for different elements when comparing a cluster to a release
     comparison_weights = {
         'album': 17,
-        'artist': 6,
+        'albumartist': 6,
         'totaltracks': 5,
         'releasecountry': 2,
         'format': 2,
@@ -59,25 +60,26 @@ class Cluster(QtCore.QObject, Item):
         return len(self.files)
 
     def add_files(self, files):
-        self.metadata['totaltracks'] += len(files)
         for file in files:
             self.metadata.length += file.metadata.length
             file._move(self)
             file.update(signal=False)
         self.files.extend(files)
+        self.metadata['totaltracks'] = len(self.files)
         self.item.add_files(files)
 
     def add_file(self, file):
-        self.metadata['totaltracks'] += 1
         self.metadata.length += file.metadata.length
         self.files.append(file)
+        self.metadata['totaltracks'] = len(self.files)
+        file._move(self)
         file.update(signal=False)
         self.item.add_file(file)
 
     def remove_file(self, file):
-        self.metadata['totaltracks'] -= 1
         self.metadata.length -= file.metadata.length
         self.files.remove(file)
+        self.metadata['totaltracks'] = len(self.files)
         self.item.remove_file(file)
         if not self.special and self.get_num_files() == 0:
             self.tagger.remove_cluster(self)
@@ -126,7 +128,7 @@ class Cluster(QtCore.QObject, Item):
 
     def column(self, column):
         if column == 'title':
-            return '%s (%d)' % (self.metadata['album'], self.metadata['totaltracks'])
+            return '%s (%d)' % (self.metadata['album'], len(self.files))
         elif (column == '~length' and self.special) or column == 'album':
             return ''
         elif column == '~length':
@@ -135,35 +137,6 @@ class Cluster(QtCore.QObject, Item):
             return self.metadata['albumartist']
         return self.metadata[column]
 
-    def _compare_to_release(self, release):
-        """
-        Compare cluster metadata to a MusicBrainz release. Produces a
-        probability as a linear combination of weights that the
-        cluster is a certain album.
-
-        Weights:
-          * title                = 17
-          * artist name          = 6
-          * number of tracks     = 5
-          * release country      = 2
-          * format               = 2
-
-        """
-        total = 0.0
-        parts = []
-        w = Cluster.comparison_weights
-
-        a = self.metadata['albumartist']
-        b = artist_credit_from_node(release.artist_credit[0], self.config)[0]
-        parts.append((similarity2(a, b), w["artist"]))
-        total += w["artist"]
-
-        t, p = self.metadata.compare_to_release(release, w, self.config)
-        total += t
-        parts.extend(p)
-
-        return reduce(lambda x, y: x + y[0] * y[1] / total, parts, 0.0)
-
     def _lookup_finished(self, document, http, error):
         self.lookup_task = None
 
@@ -178,20 +151,20 @@ class Cluster(QtCore.QObject, Item):
             return
 
         # multiple matches -- calculate similarities to each of them
-        matches = []
-        for release in releases:
-            matches.append((self._compare_to_release(release), release))
-        matches.sort(reverse=True)
-        #self.log.debug("Matches: %r", matches)
+        match = sorted((self.metadata.compare_to_release(
+            release, Cluster.comparison_weights) for release in releases),
+            reverse=True, key=itemgetter(0))[0]
 
-        if matches[0][0] < self.config.setting['cluster_lookup_threshold']:
+        if match[0] < self.config.setting['cluster_lookup_threshold']:
             self.tagger.window.set_statusbar_message(N_("No matching releases for cluster %s"), self.metadata['album'], timeout=3000)
             return
         self.tagger.window.set_statusbar_message(N_("Cluster %s identified!"), self.metadata['album'], timeout=3000)
-        self.tagger.move_files_to_album(self.files, matches[0][1].id)
+        self.tagger.move_files_to_album(self.files, match[1].id)
 
     def lookup_metadata(self):
-        """ Try to identify the cluster using the existing metadata. """
+        """Try to identify the cluster using the existing metadata."""
+        if self.lookup_task:
+            return
         self.tagger.window.set_statusbar_message(N_("Looking up the metadata for cluster %s..."), self.metadata['album'])
         self.lookup_task = self.tagger.xmlws.find_releases(self._lookup_finished,
             artist=self.metadata['albumartist'],
@@ -307,6 +280,10 @@ class ClusterList(list, Item):
     def can_browser_lookup(self):
         return False
 
+    def lookup_metadata(self):
+        for cluster in self:
+            cluster.lookup_metadata()
+
 
 class ClusterDict(object):
 
