@@ -21,15 +21,10 @@
 import __builtin__
 __builtin__.__dict__['N_'] = lambda a: a
 
-# Host for PUID submissions
-PUID_SUBMIT_HOST = "musicbrainz.org"
-PUID_SUBMIT_PORT = 80
-
-# MusicDNS client ID
-MUSICDNS_KEY = "0736ac2cd889ef77f26f6b5e3fb8a09c"
-
 # AcoustID client API key
 ACOUSTID_KEY = '0zClDiGo'
+ACOUSTID_HOST = 'api.acoustid.org'
+FPCALC_NAMES = ['fpcalc', 'pyfpcalc']
 
 # Various Artists MBID
 VARIOUS_ARTISTS_ID = '89ad4ac3-39f7-470e-963a-56509c546377'
