@@ -28,5 +28,9 @@ class DataObject(LockableObject):
         self.item = None
 
     def add_folksonomy_tag(self, name, count):
-        self.folksonomy_tags.setdefault(name, 0)
-        self.folksonomy_tags[name] += count
+        self.folksonomy_tags[name] = self.folksonomy_tags.get(name, 0) + count
+
+    @staticmethod
+    def merge_folksonomy_tags(this, that):
+        for name, count in that.iteritems():
+            this[name] = this.get(name, 0) + count
