@@ -24,11 +24,10 @@ import shutil
 import sys
 import re
 import unicodedata
-import traceback
+from operator import itemgetter
 from collections import defaultdict
 from PyQt4 import QtCore
 from picard.track import Track
-from picard.mbxml import artist_credit_from_node
 from picard.metadata import Metadata
 from picard.ui.item import Item
 from picard.script import ScriptParser
@@ -44,19 +43,12 @@ from picard.util import (
     partial,
     unaccent,
     format_time,
-    LockableObject,
     pathcmp,
     mimetype
     )
 
 
-class File(LockableObject, Item):
-
-    __id_counter = 0
-    @staticmethod
-    def new_id():
-        File.__id_counter += 1
-        return File.__id_counter
+class File(QtCore.QObject, Item):
 
     UNDEFINED = -1
     PENDING = 0
@@ -78,7 +70,6 @@ class File(LockableObject, Item):
 
     def __init__(self, filename):
         super(File, self).__init__()
-        self.id = self.new_id()
         self.filename = filename
         self.base_filename = os.path.basename(filename)
         self._state = File.UNDEFINED
@@ -86,17 +77,16 @@ class File(LockableObject, Item):
         self.error = None
 
         self.orig_metadata = Metadata()
-        self.user_metadata = Metadata()
-        self.saved_metadata = Metadata()
-        self.metadata = self.user_metadata
+        self.metadata = Metadata()
 
         self.similarity = 1.0
         self.parent = None
 
         self.lookup_task = None
+        self.item = None
 
     def __repr__(self):
-        return '<File #%d %r>' % (self.id, self.base_filename)
+        return '<File %r>' % self.base_filename
 
     def load(self, next):
         self.tagger.load_queue.put((
@@ -119,13 +109,11 @@ class File(LockableObject, Item):
         return self
 
     def _copy_loaded_metadata(self, metadata):
-        filename, extension = os.path.splitext(self.base_filename)
-        self.metadata.copy(metadata)
-        self.metadata['~extension'] = extension[1:].lower()
-        self.metadata['~length'] = format_time(self.metadata.length)
-        if 'title' not in self.metadata:
-            self.metadata['title'] = filename
-        if 'tracknumber' not in self.metadata:
+        filename, _ = os.path.splitext(self.base_filename)
+        metadata['~length'] = format_time(metadata.length)
+        if 'title' not in metadata:
+            metadata['title'] = filename
+        if 'tracknumber' not in metadata:
             match = re.match("(?:track)?\s*(?:no|nr)?\s*(\d+)", filename, re.I)
             if match:
                 try:
@@ -133,18 +121,26 @@ class File(LockableObject, Item):
                 except ValueError:
                     pass
                 else:
-                    self.metadata['tracknumber'] = str(tracknumber)
-        self.orig_metadata.copy(self.metadata)
+                    metadata['tracknumber'] = str(tracknumber)
+        self.orig_metadata.copy(metadata)
+        self.metadata = metadata
 
     def copy_metadata(self, metadata):
-        exceptions = ['musicip_puid', 'acoustid_id']
-        if self.config.setting['preserved_tags']:
-            exceptions.extend(re.split(r'\s+', self.config.setting['preserved_tags'].strip()))
-        for tag in exceptions:
-            self.saved_metadata[tag] = self.metadata[tag]
+        acoustid = self.metadata["acoustid_id"]
+        preserve = self.config.setting["preserved_tags"].strip() + " ".join([
+                "~bitrate", "~bits_per_sample", "~format", "~channels",
+                "~filename", "~dirname", "~extension"])
+        saved_metadata = {}
+
+        for tag in re.split(r"\s+", preserve):
+            values = self.orig_metadata.getall(tag)
+            if values:
+                saved_metadata[tag] = values
         self.metadata.copy(metadata)
-        for tag in exceptions:
-            self.metadata[tag] = self.saved_metadata.pop(tag)
+        for tag, values in saved_metadata.iteritems():
+            self.metadata.set(tag, values)
+
+        self.metadata["acoustid_id"] = acoustid
 
     def has_error(self):
         return self.state == File.ERROR
@@ -220,8 +216,8 @@ class File(LockableObject, Item):
             self.base_filename = os.path.basename(new_filename)
             length = self.orig_metadata.length
             temp_info = {}
-            for info in ('~#bitrate', '~#sample_rate', '~#channels',
-                         '~#bits_per_sample', '~format', '~extension'):
+            for info in ('~bitrate', '~sample_rate', '~channels',
+                         '~bits_per_sample', '~format'):
                 temp_info[info] = self.orig_metadata[info]
             if self.config.setting["clear_existing_tags"]:
                 self.orig_metadata.copy(self.metadata)
@@ -233,6 +229,7 @@ class File(LockableObject, Item):
                 self.orig_metadata[k] = v
             self.error = None
             self.clear_pending()
+            self._add_path_to_metadata(self.orig_metadata)
         return self, old_filename, new_filename
 
     def _save(self, filename, metadata, settings):
@@ -252,13 +249,13 @@ class File(LockableObject, Item):
                 metadata[name] = sanitize_filename(metadata[name])
         format = format.replace("\t", "").replace("\n", "")
         filename = ScriptParser().eval(format, metadata, self)
-        # replace incompatible characters
-        if settings["windows_compatible_filenames"] or sys.platform == "win32":
-            filename = replace_win32_incompat(filename)
         if settings["ascii_filenames"]:
             if isinstance(filename, unicode):
                 filename = unaccent(filename)
             filename = replace_non_ascii(filename)
+        # replace incompatible characters
+        if settings["windows_compatible_filenames"] or sys.platform == "win32":
+            filename = replace_win32_incompat(filename)
         # remove null characters
         filename = filename.replace("\x00", "")
         return filename
@@ -286,11 +283,11 @@ class File(LockableObject, Item):
                     new_filename = new_filename.replace('./', '_/').replace('.\\', '_\\')
                 # replace . at the beginning of file and directory names
                 new_filename = new_filename.replace('/.', '/_').replace('\\.', '\\_')
-                if new_filename[0] == '.':
+                if new_filename and new_filename[0] == '.':
                     new_filename = '_' + new_filename[1:]
                 # Fix for precomposed characters on OSX
                 if sys.platform == "darwin":
-                    new_filename = unicodedata.normalize("NFD", new_filename)
+                    new_filename = unicodedata.normalize("NFD", unicode(new_filename))
         return os.path.realpath(os.path.join(new_dirname, new_filename + ext.lower()))
 
     def _rename(self, old_filename, metadata, settings):
@@ -333,7 +330,10 @@ class File(LockableObject, Item):
             settings["cover_image_filename"], dirname, metadata, settings)
         overwrite = settings["save_images_overwrite"]
         counters = defaultdict(lambda: 0)
-        for mime, data, filename in metadata.images:
+        for image in metadata.images:
+            filename = image["filename"]
+            data = image["data"]
+            mime = image["mime"]
             if filename is None:
                 filename = default_filename
             else:
@@ -364,14 +364,13 @@ class File(LockableObject, Item):
         new_path = encode_filename(os.path.dirname(new_filename))
         patterns = encode_filename(settings["move_additional_files_pattern"])
         patterns = filter(bool, [p.strip() for p in patterns.split()])
-        files = []
         for pattern in patterns:
             # FIXME glob1 is not documented, maybe we need our own implemention?
             for old_file in glob.glob1(old_path, pattern):
                 new_file = os.path.join(new_path, old_file)
                 old_file = os.path.join(old_path, old_file)
                 # FIXME we shouldn't do this from a thread!
-                if self.tagger.get_file_by_filename(decode_filename(old_file)):
+                if self.tagger.files.get(decode_filename(old_file)):
                     self.log.debug("File loaded in the tagger, not moving %r", old_file)
                     continue
                 self.log.debug("Moving %r to %r", old_file, new_file)
@@ -381,7 +380,6 @@ class File(LockableObject, Item):
         if from_parent and self.parent:
             self.log.debug("Removing %r from %r", self, self.parent)
             self.parent.remove_file(self)
-        self.tagger.puidmanager.remove(self.metadata['musicip_puid'])
         self.tagger.acoustidmanager.remove(self)
         self.state = File.REMOVED
 
@@ -389,13 +387,12 @@ class File(LockableObject, Item):
         if parent != self.parent:
             self.log.debug("Moving %r from %r to %r", self, self.parent, parent)
             self.clear_lookup_task()
-            self.tagger._ofa.stop_analyze(self)
+            self.tagger._acoustid.stop_analyze(file)
             if self.parent:
                 self.clear_pending()
                 self.parent.remove_file(self)
             self.parent = parent
             self.parent.add_file(self)
-            self.tagger.puidmanager.update(self.metadata['musicip_puid'], self.metadata['musicbrainz_trackid'])
             self.tagger.acoustidmanager.update(self, self.metadata['musicbrainz_trackid'])
 
     def _move(self, parent):
@@ -404,7 +401,6 @@ class File(LockableObject, Item):
             if self.parent:
                 self.parent.remove_file(self)
             self.parent = parent
-            self.tagger.puidmanager.update(self.metadata['musicip_puid'], self.metadata['musicbrainz_trackid'])
             self.tagger.acoustidmanager.update(self, self.metadata['musicbrainz_trackid'])
 
     def supports_tag(self, name):
@@ -415,8 +411,8 @@ class File(LockableObject, Item):
         return self.similarity == 1.0 and self.state == File.NORMAL
 
     def update(self, signal=True):
-        names = set(self.metadata._items.keys())
-        names.update(self.orig_metadata._items.keys())
+        names = set(self.metadata.keys())
+        names.update(self.orig_metadata.keys())
         clear_existing_tags = self.config.setting["clear_existing_tags"]
         for name in names:
             if not name.startswith('~') and self.supports_tag(name):
@@ -437,8 +433,6 @@ class File(LockableObject, Item):
             self.log.debug("Updating file %r", self)
             if self.item:
                 self.item.update()
-            if isinstance(self.parent, Track):
-                self.parent.update()
 
     def can_save(self):
         """Return if this object can be saved."""
@@ -469,21 +463,21 @@ class File(LockableObject, Item):
         if hasattr(file.info, 'length'):
             metadata.length = int(file.info.length * 1000)
         if hasattr(file.info, 'bitrate') and file.info.bitrate:
-            metadata['~#bitrate'] = file.info.bitrate / 1000.0
+            metadata['~bitrate'] = file.info.bitrate / 1000.0
         if hasattr(file.info, 'sample_rate') and file.info.sample_rate:
-            metadata['~#sample_rate'] = file.info.sample_rate
+            metadata['~sample_rate'] = file.info.sample_rate
         if hasattr(file.info, 'channels') and file.info.channels:
-            metadata['~#channels'] = file.info.channels
+            metadata['~channels'] = file.info.channels
         if hasattr(file.info, 'bits_per_sample') and file.info.bits_per_sample:
-            metadata['~#bits_per_sample'] = file.info.bits_per_sample
+            metadata['~bits_per_sample'] = file.info.bits_per_sample
         metadata['~format'] = self.__class__.__name__.replace('File', '')
         self._add_path_to_metadata(metadata)
 
     def _add_path_to_metadata(self, metadata):
         metadata['~dirname'] = os.path.dirname(self.filename)
-        filename = os.path.basename(self.filename)
-        metadata['~filename'], metadata['~extension'] = os.path.splitext(
-                os.path.basename(self.filename))
+        filename, extension = os.path.splitext(os.path.basename(self.filename))
+        metadata['~filename'] = filename
+        metadata['~extension'] = extension.lower()[1:]
 
     def get_state(self):
         return self._state
@@ -501,7 +495,7 @@ class File(LockableObject, Item):
         self._state = state
         if update:
             self.update()
-        self.tagger.emit(QtCore.SIGNAL("file_state_changed"), File.num_pending_files)
+        self.tagger.file_state_changed.emit(File.num_pending_files)
 
     state = property(get_state, set_state)
 
@@ -511,60 +505,6 @@ class File(LockableObject, Item):
             return self.base_filename
         return m[column]
 
-    def _compare_to_track(self, track):
-        """
-        Compare file metadata to a MusicBrainz track.
-
-        Weigths:
-          * title                = 13
-          * artist name          = 4
-          * release name         = 5
-          * length               = 10
-          * number of tracks     = 4
-          * album type           = 20
-          * release country      = 2
-          * format               = 2
-
-        """
-        total = 0.0
-        parts = []
-        w = self.comparison_weights
-
-        if 'title' in self.metadata:
-            a = self.metadata['title']
-            b = track.title[0].text
-            parts.append((similarity2(a, b), w["title"]))
-            total += w["title"]
-
-        if 'artist' in self.metadata:
-            a = self.metadata['artist']
-            b = artist_credit_from_node(track.artist_credit[0], self.config)[0]
-            parts.append((similarity2(a, b), w["artist"]))
-            total += w["artist"]
-
-        a = self.metadata.length
-        if a > 0 and 'length' in track.children:
-            b = int(track.length[0].text)
-            score = 1.0 - min(abs(a - b), 30000) / 30000.0
-            parts.append((score, w["length"]))
-            total += w["length"]
-
-        releases = []
-        if "release_list" in track.children and "release" in track.release_list[0].children:
-            releases = track.release_list[0].release
-
-        if not releases:
-            return (total, None)
-
-        scores = []
-        for release in releases:
-            t, p = self.metadata.compare_to_release(release, w, self.config)
-            total_ = total + t
-            parts_ = list(parts) + p
-            scores.append((reduce(lambda x, y: x + y[0] * y[1] / total_, parts_, 0.0), release.id))
-
-        return max(scores, key=lambda x: x[0])
-
     def _lookup_finished(self, lookuptype, document, http, error):
         self.lookup_task = None
 
@@ -575,12 +515,8 @@ class File(LockableObject, Item):
             m = document.metadata[0]
             if lookuptype == "metadata":
                 tracks = m.recording_list[0].recording
-            elif lookuptype == "puid":
-                tracks = m.puid[0].recording_list[0].recording
             elif lookuptype == "acoustid":
-                tracks = m.puid[0].recording_list[0].recording
-            elif lookuptype == "trackid":
-                tracks = m.recording
+                tracks = m.acoustid[0].recording_list[0].recording
         except (AttributeError, IndexError):
             tracks = None
 
@@ -591,50 +527,43 @@ class File(LockableObject, Item):
             return
 
         # multiple matches -- calculate similarities to each of them
-        matches = []
-        for track in tracks:
-            score, release = self._compare_to_track(track)
-            matches.append((score, track, release))
-        matches.sort(reverse=True)
-        #self.log.debug("Track matches: %r", matches)
+        match = sorted((self.metadata.compare_to_track(
+            track, self.comparison_weights) for track in tracks),
+            reverse=True, key=itemgetter(0))[0]
 
-        if lookuptype != 'puid' and lookuptype != 'acoustid':
+        if lookuptype != 'acoustid':
             threshold = self.config.setting['file_lookup_threshold']
-            if matches[0][0] < threshold:
+            if match[0] < threshold:
                 self.tagger.window.set_statusbar_message(N_("No matching tracks above the threshold for file %s"), self.filename, timeout=3000)
                 self.clear_pending()
                 return
         self.tagger.window.set_statusbar_message(N_("File %s identified!"), self.filename, timeout=3000)
         self.clear_pending()
 
-        albumid = matches[0][2]
-        track = matches[0][1]
-        if lookuptype == 'puid':
-            self.tagger.puidmanager.add(self.metadata['musicip_puid'], track.id)
-        elif lookuptype == 'acoustid':
+        rg, release, track = match[1:]
+        if lookuptype == 'acoustid':
             self.tagger.acoustidmanager.add(self, track.id)
-        if albumid:
-            self.tagger.move_file_to_track(self, albumid, track.id)
+        if release:
+            self.tagger.get_release_group_by_id(rg.id).loaded_albums.add(release.id)
+            self.tagger.move_file_to_track(self, release.id, track.id)
         else:
             self.tagger.move_file_to_nat(self, track.id, node=track)
 
-    def lookup_puid(self, puid):
-        """ Try to identify the file using the PUID. """
-        self.tagger.window.set_statusbar_message(N_("Looking up the PUID for file %s..."), self.filename)
-        self.clear_lookup_task()
-        self.lookup_task = self.tagger.xmlws.lookup_puid(puid, partial(self._lookup_finished, 'puid'))
-
     def lookup_metadata(self):
-        """ Try to identify the file using the existing metadata. """
+        """Try to identify the file using the existing metadata."""
+        if self.lookup_task:
+            return
         self.tagger.window.set_statusbar_message(N_("Looking up the metadata for file %s..."), self.filename)
         self.clear_lookup_task()
+        metadata = self.metadata
         self.lookup_task = self.tagger.xmlws.find_tracks(partial(self._lookup_finished, 'metadata'),
-            track=self.metadata.get('title', ''),
-            artist=self.metadata.get('artist', ''),
-            release=self.metadata.get('album', ''),
-            tnum=self.metadata.get('tracknumber', ''),
-            tracks=self.metadata.get('totaltracks', ''),
-            qdur=str(self.metadata.length / 2000),
+            track=metadata['title'],
+            artist=metadata['artist'],
+            release=metadata['album'],
+            tnum=metadata['tracknumber'],
+            tracks=metadata['totaltracks'],
+            qdur=str(metadata.length / 2000),
+            isrc=metadata['isrc'],
             limit=25)
 
     def clear_lookup_task(self):
@@ -643,10 +572,9 @@ class File(LockableObject, Item):
             self.lookup_task = None
 
     def set_pending(self):
-        if self.state == File.REMOVED:
-            return
-        self.state = File.PENDING
-        self.update()
+        if self.state != File.REMOVED:
+            self.state = File.PENDING
+            self.update()
 
     def clear_pending(self):
         if self.state == File.PENDING:
