@@ -135,9 +135,10 @@ class MP4File(File):
                 metadata["totaldiscs"] = str(values[0][1])
             elif name == "covr":
                 for value in values:
-                    if value.format == value.FORMAT_JPEG:
+                    value = MP4Cover(value)
+                    if value.imageformat == value.FORMAT_JPEG:
                         metadata.add_image("image/jpeg", value)
-                    elif value.format == value.FORMAT_PNG:
+                    elif value.imageformat == value.FORMAT_PNG:
                         metadata.add_image("image/png", value)
 
         self._info(metadata, file)
@@ -186,11 +187,14 @@ class MP4File(File):
 
         if settings['save_images_to_tags']:
             covr = []
-            for mime, data, _fname in metadata.images:
+            for image in metadata.images:
+                if self.config.setting["save_only_front_images_to_tags"] and image["type"] != "front":
+                    continue
+                mime = image["mime"]
                 if mime == "image/jpeg":
-                    covr.append(MP4Cover(data, MP4Cover.FORMAT_JPEG))
+                    covr.append(MP4Cover(image["data"], MP4Cover.FORMAT_JPEG))
                 elif mime == "image/png":
-                    covr.append(MP4Cover(data, MP4Cover.FORMAT_PNG))
+                    covr.append(MP4Cover(image["data"], MP4Cover.FORMAT_PNG))
             if covr:
                 file.tags["covr"] = covr
 
