@@ -51,7 +51,7 @@ shutil.copystat = _patched_shutil_copystat
 import picard.resources
 import picard.plugins
 
-from picard import musicdns, version_string, log, acoustid
+from picard import version_string, log, acoustid
 from picard.album import Album, NatAlbum
 from picard.browser.browser import BrowserIntegration
 from picard.browser.filelookup import FileLookup
@@ -61,9 +61,11 @@ from picard.disc import Disc
 from picard.file import File
 from picard.formats import open as open_file
 from picard.track import Track, NonAlbumTrack
+from picard.releasegroup import ReleaseGroup
+from picard.collection import load_user_collections
 from picard.ui.mainwindow import MainWindow
+from picard.ui.itemviews import BaseTreeView
 from picard.plugin import PluginManager
-from picard.puidmanager import PUIDManager
 from picard.acoustidmanager import AcoustIDManager
 from picard.util import (
     decode_filename,
@@ -79,6 +81,7 @@ from picard.webservice import XmlWebService
 class Tagger(QtGui.QApplication):
 
     file_state_changed = QtCore.pyqtSignal(int)
+    listen_port_changed = QtCore.pyqtSignal(int)
     cluster_added = QtCore.pyqtSignal(Cluster)
     cluster_removed = QtCore.pyqtSignal(Cluster)
     album_added = QtCore.pyqtSignal(Album)
@@ -109,8 +112,8 @@ class Tagger(QtGui.QApplication):
         self.other_queue = queue.Queue()
 
         threads = self.thread_pool.threads
-        threads.append(thread.Thread(self.thread_pool, self.load_queue))
-        threads.append(thread.Thread(self.thread_pool, self.load_queue))
+        for i in range(4):
+            threads.append(thread.Thread(self.thread_pool, self.load_queue))
         threads.append(thread.Thread(self.thread_pool, self.save_queue))
         threads.append(thread.Thread(self.thread_pool, self.other_queue))
         threads.append(thread.Thread(self.thread_pool, self.other_queue))
@@ -149,9 +152,9 @@ class Tagger(QtGui.QApplication):
 
         self.xmlws = XmlWebService()
 
+        load_user_collections()
+
         # Initialize fingerprinting
-        self._ofa = musicdns.OFA()
-        self._ofa.init()
         self._acoustid = acoustid.AcoustIDClient()
         self._acoustid.init()
 
@@ -166,13 +169,13 @@ class Tagger(QtGui.QApplication):
         else:
             self.pluginmanager.load_plugindir(os.path.join(os.path.dirname(__file__), "plugins"))
 
-        self.puidmanager = PUIDManager()
         self.acoustidmanager = AcoustIDManager()
         self.browser_integration = BrowserIntegration()
 
         self.files = {}
         self.clusters = ClusterList()
         self.albums = {}
+        self.release_groups = {}
         self.mbid_redirects = {}
         self.unmatched_files = UnmatchedFiles()
         self.nats = None
@@ -205,9 +208,10 @@ class Tagger(QtGui.QApplication):
 
     def setup_gettext(self, localedir):
         """Setup locales, load translations, install gettext functions."""
-        if self.config.setting["ui_language"]:
+        ui_language = self.config.setting["ui_language"]
+        if ui_language:
             os.environ['LANGUAGE'] = ''
-            os.environ['LANG'] = self.config.setting["ui_language"]
+            os.environ['LANG'] = ui_language
         if sys.platform == "win32":
             try:
                 locale.setlocale(locale.LC_ALL, os.environ["LANG"])
@@ -220,6 +224,13 @@ class Tagger(QtGui.QApplication):
             except:
                 pass
         else:
+            if sys.platform == "darwin" and not ui_language:
+                try:
+                    import Foundation
+                    defaults = Foundation.NSUserDefaults.standardUserDefaults()
+                    os.environ["LANG"] = defaults.objectForKey_("AppleLanguages")[0]
+                except:
+                    pass
             try:
                 locale.setlocale(locale.LC_ALL, "")
             except:
@@ -273,7 +284,6 @@ class Tagger(QtGui.QApplication):
 
     def exit(self):
         self.stopping = True
-        self._ofa.done()
         self._acoustid.done()
         self.thread_pool.stop()
         self.browser_integration.stop()
@@ -309,13 +319,13 @@ class Tagger(QtGui.QApplication):
             return 1
         return QtGui.QApplication.event(self, event)
 
-    def _file_loaded(self, result=None, error=None):
+    def _file_loaded(self, target, result=None, error=None):
         file = result
         if file is not None and error is None and not file.has_error():
-            puid = file.metadata['musicip_puid']
             trackid = file.metadata['musicbrainz_trackid']
-            self.puidmanager.add(puid, trackid)
-            if not self.config.setting["ignore_file_mbids"]:
+            if target is not None:
+                self.move_files([file], target)
+            elif not self.config.setting["ignore_file_mbids"]:
                 albumid = file.metadata['musicbrainz_albumid']
                 if mbid_validate(albumid):
                     if mbid_validate(trackid):
@@ -329,7 +339,19 @@ class Tagger(QtGui.QApplication):
             elif self.config.setting['analyze_new_files'] and file.can_analyze():
                 self.analyze([file])
 
-    def add_files(self, filenames):
+    def move_files(self, files, target):
+        if isinstance(target, (Track, Cluster)):
+            for file in files:
+                file.move(target)
+        elif isinstance(target, File):
+            for file in files:
+                file.move(target.parent)
+        elif isinstance(target, Album):
+            self.move_files_to_album(files, album=target)
+        elif isinstance(target, ClusterList):
+            self.cluster(files)
+
+    def add_files(self, filenames, target=None):
         """Add files to the tagger."""
         self.log.debug("Adding files %r", filenames)
         new_files = []
@@ -341,58 +363,31 @@ class Tagger(QtGui.QApplication):
                     self.files[filename] = file
                     new_files.append(file)
         if new_files:
-            self.unmatched_files.add_files(new_files)
+            if target is None or target is self.unmatched_files:
+                self.unmatched_files.add_files(new_files)
+                target = None
             for file in new_files:
-                file.load(self._file_loaded)
-
-    def process_directory_listing(self, root, queue, result=None, error=None):
-        try:
-            # Read directory listing
-            if result is not None and error is None:
-                files = []
-                directories = deque()
-                try:
-                    for path in result:
-                        path = os.path.join(root, path)
-                        if os.path.isdir(path):
-                            directories.appendleft(path)
-                        else:
-                            try:
-                                files.append(decode_filename(path))
-                            except UnicodeDecodeError:
-                                self.log.warning("Failed to decode filename: %r", path)
-                                continue
-                finally:
-                    if files:
-                        self.add_files(files)
-                    queue.extendleft(directories)
-        finally:
-            # Scan next directory in the queue
-            try:
-                path = queue.popleft()
-            except IndexError: pass
-            else:
-                self.other_queue.put((
-                    partial(os.listdir, path),
-                    partial(self.process_directory_listing, path, queue),
-                    QtCore.Qt.LowEventPriority))
+                file.load(partial(self._file_loaded, target))
 
     def add_directory(self, path):
-        path = encode_filename(path)
-        self.other_queue.put((partial(os.listdir, path),
-                              partial(self.process_directory_listing, path, deque()),
-                              QtCore.Qt.LowEventPriority))
+        walk = os.walk(unicode(path))
 
-    def get_file_by_id(self, id):
-        """Get file by a file ID."""
-        for file in self.files.itervalues():
-            if file.id == id:
-                return file
-        return None
+        def get_files():
+            try:
+                root, dirs, files = walk.next()
+            except StopIteration:
+                return None
+            else:
+                self.window.set_statusbar_message(N_("Loading directory %s"), root)
+                return (os.path.join(root, f) for f in files)
 
-    def get_file_by_filename(self, filename):
-        """Get file by a filename."""
-        return self.files.get(filename, None)
+        def process(result=None, error=None):
+            if result:
+                if error is None:
+                    self.add_files(result)
+                self.other_queue.put((get_files, process, QtCore.Qt.LowEventPriority))
+
+        process(True, False)
 
     def get_file_lookup(self):
         """Return a FileLookup object."""
@@ -423,8 +418,7 @@ class Tagger(QtGui.QApplication):
                 metadata["title"],
                 metadata["tracknumber"],
                 '' if item.is_album_like() else str(metadata.length),
-                item.filename if isinstance(item, File) else '',
-                metadata["musicip_puid"])
+                item.filename if isinstance(item, File) else '')
 
     def get_files_from_objects(self, objects, save=False):
         """Return list of files from list of albums, clusters, tracks or files."""
@@ -476,12 +470,14 @@ class Tagger(QtGui.QApplication):
                 if nat.id == id:
                     return nat
 
+    def get_release_group_by_id(self, id):
+        return self.release_groups.setdefault(id, ReleaseGroup(id))
+
     def remove_files(self, files, from_parent=True):
         """Remove files from the tagger."""
         for file in files:
             if self.files.has_key(file.filename):
                 file.clear_lookup_task()
-                self._ofa.stop_analyze(file)
                 self._acoustid.stop_analyze(file)
                 del self.files[file.filename]
                 file.remove(from_parent)
@@ -492,6 +488,8 @@ class Tagger(QtGui.QApplication):
         album.stop_loading()
         self.remove_files(self.get_files_from_objects([album]))
         del self.albums[album.id]
+        if album.release_group:
+            album.release_group.remove_album(album.id)
         if album == self.nats:
             self.nats = None
         self.album_removed.emit(album)
@@ -530,12 +528,12 @@ class Tagger(QtGui.QApplication):
         else:
             disc.lookup()
 
-    def lookup_cd(self, action=None):
+    def lookup_cd(self, action):
         """Reads CD from the selected drive and tries to lookup the DiscID on MusicBrainz."""
-        if action is None:
-            device = self.config.setting["cd_lookup_device"].split(",", 1)[0]
-        else:
+        if isinstance(action, QtGui.QAction):
             device = unicode(action.text())
+        else:
+            device = self.config.setting["cd_lookup_device"].split(",", 1)[0]
 
         disc = Disc()
         self.set_wait_cursor()
@@ -544,17 +542,6 @@ class Tagger(QtGui.QApplication):
             partial(self._lookup_disc, disc),
             QtCore.Qt.LowEventPriority))
 
-    def _lookup_puid(self, file, result=None, error=None):
-        puid = result
-        if file.state == File.PENDING:
-            if puid:
-                self.puidmanager.add(puid, None)
-                file.metadata['musicip_puid'] = puid
-                file.lookup_puid(puid)
-            else:
-                self.window.set_statusbar_message(N_("Could not find PUID for file %s"), file.filename)
-                file.clear_pending()
-
     @property
     def use_acoustid(self):
         return self.config.setting["fingerprinting_system"] == "acoustid"
@@ -566,8 +553,6 @@ class Tagger(QtGui.QApplication):
             file.set_pending()
             if self.use_acoustid:
                 self._acoustid.analyze(file, partial(file._lookup_finished, 'acoustid'))
-            else:
-                self._ofa.analyze(file, partial(self._lookup_puid, file))
 
     # =======================================================================
     #  Metadata-based lookups
@@ -575,7 +560,7 @@ class Tagger(QtGui.QApplication):
 
     def autotag(self, objects):
         for obj in objects:
-            if isinstance(obj, (File, Cluster)) and not obj.lookup_task:
+            if obj.can_autotag():
                 obj.lookup_metadata()
 
     # =======================================================================
@@ -633,9 +618,6 @@ class Tagger(QtGui.QApplication):
     def num_files(self):
         return len(self.files)
 
-    def num_pending_files(self):
-        return len([file for file in self.files.values() if file.state == File.PENDING])
-
 def help():
     print """Usage: %s [OPTIONS] [FILE] [FILE] ...
 
