@@ -18,10 +18,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtCore
 from picard.metadata import Metadata, run_track_metadata_processors
 from picard.dataobj import DataObject
-from picard.util import format_time, asciipunct, partial
+from picard.util import asciipunct, partial
 from picard.mbxml import recording_to_metadata
 from picard.script import ScriptParser
 from picard.const import VARIOUS_ARTISTS_ID
@@ -129,9 +128,9 @@ class Track(DataObject, Item):
     def _convert_folksonomy_tags_to_genre(self):
         # Combine release and track tags
         tags = dict(self.folksonomy_tags)
-        for name, count in self.album.folksonomy_tags.iteritems():
-            tags.setdefault(name, 0)
-            tags[name] += count
+        self.merge_folksonomy_tags(tags, self.album.folksonomy_tags)
+        if self.album.release_group:
+            self.merge_folksonomy_tags(tags, self.album.release_group.folksonomy_tags)
         if not tags:
             return
         # Convert counts to values from 0 to 100
