@@ -24,11 +24,11 @@ from picard.album import Album, NatAlbum
 from picard.cluster import Cluster, ClusterList, UnmatchedFiles
 from picard.file import File
 from picard.track import Track, NonAlbumTrack
-from picard.util import encode_filename, icontheme, partial, webbrowser2
+from picard.util import encode_filename, icontheme, partial
 from picard.config import Option, TextOption
 from picard.plugin import ExtensionPoint
-from picard.const import RELEASE_COUNTRIES
 from picard.ui.ratingwidget import RatingWidget
+from picard.ui.collectionmenu import CollectionMenu
 
 
 class BaseAction(QtGui.QAction):
@@ -179,12 +179,23 @@ class MainPanel(QtGui.QSplitter):
     def update_current_view(self):
         self.update_selection(self._selected_view, abs(self._selected_view - 1))
 
+    def remove(self, objects):
+        self._ignore_selection_changes = True
+        self.tagger.remove(objects)
+        self._ignore_selection_changes = False
+
+        view = self.views[self._selected_view]
+        index = view.currentIndex()
+        if index.isValid():
+            # select the current index
+            view.setCurrentIndex(index)
+        else:
+            self.update_current_view()
+
 
 class BaseTreeView(QtGui.QTreeWidget):
 
     options = [
-        TextOption("persist", "file_view_sizes", "250 40 100"),
-        TextOption("persist", "album_view_sizes", "250 40 100"),
         Option("setting", "color_modified", QtGui.QColor(QtGui.QPalette.WindowText), QtGui.QColor),
         Option("setting", "color_saved", QtGui.QColor(0, 128, 0), QtGui.QColor),
         Option("setting", "color_error", QtGui.QColor(200, 0, 0), QtGui.QColor),
@@ -261,6 +272,8 @@ class BaseTreeView(QtGui.QTreeWidget):
             menu.addAction(self.window.analyze_action)
             plugin_actions = list(_file_actions)
         elif isinstance(obj, Album):
+            if can_view_info:
+                menu.addAction(self.window.view_info_action)
             menu.addAction(self.window.browser_lookup_action)
             menu.addSeparator()
             menu.addAction(self.window.refresh_action)
@@ -269,33 +282,31 @@ class BaseTreeView(QtGui.QTreeWidget):
         menu.addAction(self.window.save_action)
         menu.addAction(self.window.remove_action)
 
+        bottom_separator = False
+
         if isinstance(obj, Album) and not isinstance(obj, NatAlbum) and obj.loaded:
             releases_menu = QtGui.QMenu(_("&Other versions"), menu)
             menu.addSeparator()
             menu.addMenu(releases_menu)
             loading = releases_menu.addAction(_('Loading...'))
             loading.setEnabled(False)
+            bottom_separator = True
 
-            def _add_other_versions():
-                releases_menu.removeAction(loading)
-                actions = []
-                for i, version in enumerate(obj.other_versions):
-                    keys = ("date", "country", "labels", "catnums", "tracks", "format")
-                    name = " / ".join([version[k] for k in keys if version[k]]).replace("&", "&&")
-                    if name == version["tracks"]:
-                        name = "%s / %s" % (_('[no release info]'), name)
-                    action = releases_menu.addAction(name)
-                    action.setCheckable(True)
-                    if obj.id == version["mbid"]:
-                        action.setChecked(True)
-                    action.triggered.connect(partial(obj.switch_release_version, version["mbid"]))
+            if len(self.selectedIndexes()) == len(MainPanel.columns):
+                def _add_other_versions():
+                    releases_menu.removeAction(loading)
+                    for version in obj.release_group.versions:
+                        action = releases_menu.addAction(version["name"])
+                        action.setCheckable(True)
+                        if obj.id == version["id"]:
+                            action.setChecked(True)
+                        action.triggered.connect(partial(obj.switch_release_version, version["id"]))
 
-            if obj.rgloaded:
-                _add_other_versions()
-            elif obj.rgid:
-                obj.release_group_loaded.connect(_add_other_versions)
-                kwargs = {"release-group": obj.rgid, "limit": 100}
-                self.tagger.xmlws.browse_releases(obj._release_group_request_finished, **kwargs)
+                _add_other_versions() if obj.release_group.loaded else \
+                    obj.release_group.load_versions(_add_other_versions)
+                releases_menu.setEnabled(True)
+            else:
+                releases_menu.setEnabled(False)
 
         if self.config.setting["enable_ratings"] and \
            len(self.window.selected_objects) == 1 and isinstance(obj, Track):
@@ -305,6 +316,12 @@ class BaseTreeView(QtGui.QTreeWidget):
             menu.addAction(action)
             menu.addSeparator()
 
+        selected_albums = [a for a in self.window.selected_objects if type(a) == Album]
+        if selected_albums:
+            if not bottom_separator:
+                menu.addSeparator()
+            menu.addMenu(CollectionMenu(selected_albums, _("Collections"), menu))
+
         if plugin_actions:
             plugin_menu = QtGui.QMenu(_("&Plugins"), menu)
             plugin_menu.setIcon(self.panel.icon_plugins)
@@ -323,6 +340,7 @@ class BaseTreeView(QtGui.QTreeWidget):
                 action_menu.addAction(action)
 
         if isinstance(obj, Cluster) or isinstance(obj, ClusterList) or isinstance(obj, Album):
+            menu.addSeparator()
             menu.addAction(self.expand_all_action)
             menu.addAction(self.collapse_all_action)
 
@@ -330,10 +348,7 @@ class BaseTreeView(QtGui.QTreeWidget):
         event.accept()
 
     def restore_state(self):
-        if self.__class__.__name__ == "FileTreeView":
-            sizes = self.config.persist["file_view_sizes"]
-        else:
-            sizes = self.config.persist["album_view_sizes"]
+        sizes = self.config.persist[self.view_sizes.name]
         header = self.header()
         sizes = sizes.split(" ")
         try:
@@ -343,24 +358,16 @@ class BaseTreeView(QtGui.QTreeWidget):
             pass
 
     def save_state(self):
-        sizes = []
-        header = self.header()
-        for i in range(self.numHeaderSections - 1):
-            sizes.append(str(self.header().sectionSize(i)))
-        sizes = " ".join(sizes)
-        if self.__class__.__name__ == "FileTreeView":
-            self.config.persist["file_view_sizes"] = sizes
-        else:
-            self.config.persist["album_view_sizes"] = sizes
+        cols = range(self.numHeaderSections - 1)
+        sizes = " ".join(str(self.header().sectionSize(i)) for i in cols)
+        self.config.persist[self.view_sizes.name] = sizes
 
     def supportedDropActions(self):
         return QtCore.Qt.CopyAction | QtCore.Qt.MoveAction
 
     def mimeTypes(self):
         """List of MIME types accepted by this view."""
-        return ["text/uri-list",
-                "application/picard.file-list",
-                "application/picard.album-list"]
+        return ["text/uri-list", "application/picard.album-list"]
 
     def dragEnterEvent(self, event):
         if event.mimeData().hasUrls():
@@ -380,74 +387,55 @@ class BaseTreeView(QtGui.QTreeWidget):
     def mimeData(self, items):
         """Return MIME data for specified items."""
         album_ids = []
-        file_ids = []
+        files = []
+        url = QtCore.QUrl.fromLocalFile
         for item in items:
             obj = item.obj
             if isinstance(obj, Album):
                 album_ids.append(str(obj.id))
             elif isinstance(obj, Track):
-                for file in obj.linked_files:
-                    file_ids.append(str(file.id))
+                files.extend(url(file.filename) for file in obj.linked_files)
             elif isinstance(obj, File):
-                file_ids.append(str(obj.id))
+                files.append(url(obj.filename))
             elif isinstance(obj, Cluster):
-                for file in obj.files:
-                    file_ids.append(str(file.id))
+                files.extend(url(file.filename) for file in obj.files)
             elif isinstance(obj, ClusterList):
-                for cluster in obj:
-                    for file in cluster.files:
-                        file_ids.append(str(file.id))
+                files.extend(url(file.filename) for cluster in obj for file in cluster.files)
         mimeData = QtCore.QMimeData()
         mimeData.setData("application/picard.album-list", "\n".join(album_ids))
-        mimeData.setData("application/picard.file-list", "\n".join(file_ids))
+        if files:
+            mimeData.setUrls(files)
         return mimeData
 
-    def drop_files(self, files, target):
-        if isinstance(target, (Track, Cluster)):
-            for file in files:
-                file.move(target)
-        elif isinstance(target, File):
-            for file in files:
-                file.move(target.parent)
-        elif isinstance(target, Album):
-            self.tagger.move_files_to_album(files, album=target)
-        elif isinstance(target, ClusterList):
-            self.tagger.cluster(files)
-
-    def drop_albums(self, albums, target):
-        files = self.tagger.get_files_from_objects(albums)
-        if isinstance(target, Cluster):
-            for file in files:
-                file.move(target)
-        elif isinstance(target, Album):
-            self.tagger.move_files_to_album(files, album=target)
-        elif isinstance(target, ClusterList):
-            self.tagger.cluster(files)
-
-    def drop_urls(self, urls, target):
-        # URL -> Unmatched Files
-        # TODO: use the drop target to move files to specific albums/tracks/clusters
+    @staticmethod
+    def drop_urls(urls, target):
         files = []
+        new_files = []
         for url in urls:
             if url.scheme() == "file" or not url.scheme():
-                filename = unicode(url.toLocalFile())
-                if os.path.isdir(encode_filename(filename)):
-                    self.tagger.add_directory(filename)
+                # Dropping a file from iTunes gives a filename with a NULL terminator
+                filename = os.path.normpath(os.path.realpath(unicode(url.toLocalFile()).rstrip("\0")))
+                file = BaseTreeView.tagger.files.get(filename)
+                if file:
+                    files.append(file)
+                elif os.path.isdir(encode_filename(filename)):
+                    BaseTreeView.tagger.add_directory(filename)
                 else:
-                    files.append(filename)
-            elif url.scheme() == "http":
+                    new_files.append(filename)
+            elif url.scheme() in ("http", "https"):
                 path = unicode(url.path())
                 match = re.search(r"/(release|recording)/([0-9a-z\-]{36})", path)
-                if not match:
-                    continue
-                entity = match.group(1)
-                mbid = match.group(2)
-                if entity == "release":
-                    self.tagger.load_album(mbid)
-                elif entity == "recording":
-                    self.tagger.load_nat(mbid)
+                if match:
+                    entity = match.group(1)
+                    mbid = match.group(2)
+                    if entity == "release":
+                        BaseTreeView.tagger.load_album(mbid)
+                    elif entity == "recording":
+                        BaseTreeView.tagger.load_nat(mbid)
         if files:
-            self.tagger.add_files(files)
+            BaseTreeView.tagger.move_files(files, target)
+        if new_files:
+            BaseTreeView.tagger.add_files(new_files, target=target)
 
     def dropEvent(self, event):
         return QtGui.QTreeView.dropEvent(self, event)
@@ -470,25 +458,21 @@ class BaseTreeView(QtGui.QTreeWidget):
                 target = self.tagger.unmatched_files
             self.drop_urls(urls, target)
             handled = True
-        # application/picard.file-list
-        files = data.data("application/picard.file-list")
-        if files:
-            files = [self.tagger.get_file_by_id(int(file_id)) for file_id in str(files).split("\n")]
-            self.drop_files(files, target)
-            handled = True
         # application/picard.album-list
         albums = data.data("application/picard.album-list")
         if albums:
             if isinstance(self, FileTreeView) and target is None:
                 target = self.tagger.unmatched_files
             albums = [self.tagger.load_album(id) for id in str(albums).split("\n")]
-            self.drop_albums(albums, target)
+            self.tagger.move_files(self.tagger.get_files_from_objects(albums), target)
             handled = True
         return handled
 
     def activate_item(self, index):
         obj = self.itemFromIndex(index).obj
-        if obj.can_view_info():
+        # Double-clicking albums should expand them. The album info can be
+        # viewed by using the toolbar button.
+        if not isinstance(obj, Album) and obj.can_view_info():
             self.window.view_info()
 
     def add_cluster(self, cluster, parent_item=None):
@@ -501,13 +485,6 @@ class BaseTreeView(QtGui.QTreeWidget):
         else:
             cluster_item.add_files(cluster.files)
 
-    def currentChanged(self, current, previous):
-        QtGui.QTreeWidget.currentChanged(self, current, previous)
-        item = self.itemFromIndex(current)
-        if item and not item.isSelected():
-            self.setCurrentItem(item)
-            self.setCurrentIndex(current)
-
     def moveCursor(self, action, modifiers):
         if action in (QtGui.QAbstractItemView.MoveUp, QtGui.QAbstractItemView.MoveDown):
             item = self.currentItem()
@@ -518,6 +495,8 @@ class BaseTreeView(QtGui.QTreeWidget):
 
 class FileTreeView(BaseTreeView):
 
+    view_sizes = TextOption("persist", "file_view_sizes", "250 40 100")
+
     def __init__(self, window, parent=None):
         BaseTreeView.__init__(self, window, parent)
         self.unmatched_files = ClusterItem(self.tagger.unmatched_files, False, self)
@@ -536,6 +515,8 @@ class FileTreeView(BaseTreeView):
 
 class AlbumTreeView(BaseTreeView):
 
+    view_sizes = TextOption("persist", "album_view_sizes", "250 40 100")
+
     def __init__(self, window, parent=None):
         BaseTreeView.__init__(self, window, parent)
         self.tagger.album_added.connect(self.add_album)
@@ -652,7 +633,7 @@ class TrackItem(TreeItem):
         track = self.obj
         if track.num_linked_files == 1:
             file = track.linked_files[0]
-            file.item = None
+            file.item = self
             color = TrackItem.track_colors[file.state]
             bgcolor = get_match_color(file.similarity, TreeItem.base_color)
             icon = FileItem.decide_file_icon(file)
