@@ -20,6 +20,7 @@
 from picard.config import IntOption, TextOption, BoolOption, PasswordOption
 from picard.ui.options import OptionsPage, register_options_page
 from picard.ui.ui_options_general import Ui_GeneralOptionsPage
+from picard.util import rot13
 
 
 class GeneralOptionsPage(OptionsPage):
@@ -36,6 +37,7 @@ class GeneralOptionsPage(OptionsPage):
         TextOption("setting", "username", ""),
         PasswordOption("setting", "password", ""),
         BoolOption("setting", "analyze_new_files", False),
+        BoolOption("setting", "ignore_file_mbids", False),
     ]
 
     def __init__(self, parent=None):
@@ -44,8 +46,6 @@ class GeneralOptionsPage(OptionsPage):
         self.ui.setupUi(self)
         mirror_servers = [
             "musicbrainz.org",
-            "nl.musicbrainz.org",
-            "de.musicbrainz.org",
             ]
         self.ui.server_host.addItems(sorted(mirror_servers))
 
@@ -55,14 +55,16 @@ class GeneralOptionsPage(OptionsPage):
         self.ui.username.setText(self.config.setting["username"])
         self.ui.password.setText(self.config.setting["password"])
         self.ui.analyze_new_files.setChecked(self.config.setting["analyze_new_files"])
+        self.ui.ignore_file_mbids.setChecked(self.config.setting["ignore_file_mbids"])
 
     def save(self):
-        self.config.setting["server_host"] = unicode(self.ui.server_host.currentText())
+        self.config.setting["server_host"] = unicode(self.ui.server_host.currentText()).strip()
         self.config.setting["server_port"] = self.ui.server_port.value()
         self.config.setting["username"] = unicode(self.ui.username.text())
         # trivially encode the password, just to not make it so apparent
-        self.config.setting["password"] = unicode(self.ui.password.text()).encode('rot13')
+        self.config.setting["password"] = rot13(unicode(self.ui.password.text()))
         self.config.setting["analyze_new_files"] = self.ui.analyze_new_files.isChecked()
+        self.config.setting["ignore_file_mbids"] = self.ui.ignore_file_mbids.isChecked()
 
 
 register_options_page(GeneralOptionsPage)
