@@ -52,36 +52,6 @@ def asciipunct(s):
     return s
 
 
-def needs_read_lock(func):
-    """Adds a read lock around ``func``.
-
-    This decorator should be used only on ``LockableObject`` methods."""
-    def locked(self, *args, **kwargs):
-        self.lock_for_read()
-        try:
-            return func(self, *args, **kwargs)
-        finally:
-            self.unlock()
-    locked.__doc__ = func.__doc__
-    locked.__name__ = func.__name__
-    return locked
-
-
-def needs_write_lock(func):
-    """Adds a write lock around ``func``.
-
-    This decorator should be used only on ``LockableObject`` methods."""
-    def locked(self, *args, **kwargs):
-        self.lock_for_write()
-        try:
-            return func(self, *args, **kwargs)
-        finally:
-            self.unlock()
-    locked.__doc__ = func.__doc__
-    locked.__name__ = func.__name__
-    return locked
-
-
 class LockableObject(QtCore.QObject):
     """Read/write lockable object."""
 
@@ -102,24 +72,6 @@ class LockableObject(QtCore.QObject):
         self.__lock.unlock()
 
 
-class LockableDict(dict):
-
-    def __init__(self):
-        self.__lock = QtCore.QReadWriteLock()
-
-    def lock_for_read(self):
-        """Lock the object for read operations."""
-        self.__lock.lockForRead()
-
-    def lock_for_write(self):
-        """Lock the object for write operations."""
-        self.__lock.lockForWrite()
-
-    def unlock(self):
-        """Unlock the object."""
-        self.__lock.unlock()
-
-
 _io_encoding = sys.getfilesystemencoding()
 
 #The following was adapted from k3b's source code:
@@ -322,11 +274,9 @@ def find_existing_path(path):
     return decode_filename(path)
 
 
-def find_executable(name):
+def find_executable(*executables):
     if sys.platform == 'win32':
-        executables = [name + '.exe']
-    else:
-        executables = [name]
+        executables = [e + '.exe' for e in executables]
     paths = [os.path.dirname(sys.executable)] if sys.executable else []
     paths += os.environ.get('PATH', '').split(os.pathsep)
     for path in paths:
